/*
**++
**  FACILITY:
**      NEWSFILES
**
**  ABSTRACT:
**      This module performs the IO functions for NEWS
**
**  AUTHOR:
**      Geoff Huston
**
**  COPYRIGHT:
**      Copyright  1988,1989,1990,1991
**
**  VERSION HISTORY:
**	V6.0-1	 4-Nov-1990	glass@mgi.com
**	  - open_groups: access to restricted newsgroups correction
**	V6.0-4	25-May-1991	volz@process.com
**	  - Added TCPware support
**	V6.1	 6-Feb-1992	rankin@eql.caltech.edu
**	  - Lint cleanup
**	V6.1	15-Feb-1992	rankin@eql.caltech.edu
**	  - Optimizations to speed up startup, particularly for non-ANU-NEWS
**	    servers which may provide groups out of alphabetical order.
**--
*/

#if vax11c
#module NEWSFILES "V6.1"
#endif

#define _NEWSFILES_C

#include "newsinclude.h"
#include "newsextern.h"

int kid_valid = 0, nntp_process = 0;

int rms$errno = 0;

extern int profile_flags;

#if !NNTP_CLIENT_ONLY
static
struct XABKEY xabkey_1,
              xabkey_2,
              xabkey_3,
              xabkey_4;

static
struct XABPRO xabpro_1,
              xabpro_2;
#endif

struct rms_file *rms_open(nam,default_nam)
  char *nam, *default_nam;
{
  struct rms_file *i;

  i = (struct rms_file *) malloc(sizeof *i);
  i->f = cc$rms_fab;
  i->f.fab$b_fac = FAB$M_GET;
  i->f.fab$l_fna = nam;
  i->f.fab$b_fns = strlen(nam);
  if (default_nam) {
    i->f.fab$l_dna = default_nam;
    i->f.fab$b_dns = strlen(default_nam);
    }
  i->r = cc$rms_rab;
  i->r.rab$l_fab = &(i->f);

  if (sys$open(&(i->f)) & 1) {
    if (sys$connect(&(i->r)) & 1)
      return(i);
    else {
      rms$errno = i->f.fab$l_stv;
      sys$close(&(i->f));
      free(i);
      return ((struct rms_file *) 0);
      }
    }
  else {
    rms$errno = i->f.fab$l_stv;
    free(i);
    return((struct rms_file *) 0);
    }
}

int rms_get(b,size,i)
  char *b;
  int size;
  struct rms_file *i;
{
  i->r.rab$l_ubf = b;
  i->r.rab$w_usz = size;
  status = sys$get(&(i->r));
  if (status & 1) b[i->r.rab$w_rsz] = '\0';
  return(status & 1);
}

struct rms_file *
rms_create(nam,prot)
  char *nam;
  unsigned int prot;
{
  struct rms_file *i;

  i = (struct rms_file *) malloc(sizeof *i);
  i->f = cc$rms_fab;
  i->f.fab$b_fac = FAB$M_PUT;
  i->f.fab$l_fna = nam;
  i->f.fab$b_fns = strlen(nam);
  i->f.fab$b_org = FAB$C_SEQ;
  i->f.fab$b_rat = FAB$M_CR;
  i->f.fab$b_rfm = FAB$C_VAR;
  i->f.fab$l_alq = 4;
  i->f.fab$w_deq = 32;
  i->f.fab$l_fop |= FAB$M_TEF;
  i->f.fab$l_xab = (char *) &(i->p);
  i->p = cc$rms_xabpro;
  i->p.xab$w_pro = prot;

  i->r = cc$rms_rab;
  i->r.rab$l_fab = &(i->f);

  sysprv();
  if (sys$create(&(i->f)) & 1) {
    if (sys$connect(&(i->r)) & 1) {
      nosysprv();
      return(i);
      }
    else {
      sys$close(&(i->f));
      free(i);
      nosysprv();
      return((struct rms_file *) 0);
      }
    }
  else {
    free(i);
    nosysprv();
    return ((struct rms_file *) 0);
    }
}

int rms_put(s,i)
  char *s;
  struct rms_file *i;
{
  i->r.rab$l_rbf = s;
  i->r.rab$w_rsz = strlen(s);
  return(sys$put(&(i->r)) & 1);
}

void
rms_close(i)
  struct rms_file *i;
{
  sys$close(&(i->f));
  free(i);
}

/*
 *  openfiles
 *
 *  Open the item and uaf files for sharing
 */

void openfiles(nntp_sts)
  int nntp_sts;
{
  char *in_line = 0, *gotenv, *cp, tmp[256];

  if (newsrc) fseek(newsrc,0,0);
  else newsrc = fopen(news_register,"r","mbc=40");

  if (   newsrc
      && (in_line = fgetl(newsrc)) != 0
      && *in_line == '~'
      && (cp = strchr(in_line+1,'~')) != 0) {
    char newsrcnode[132];

    if (nntp_sts < 0) {
      nntp_client = 0;
      fclose(newsrc); newsrc = 0;
      strcpy(news_register,"SYS$LOGIN:NEWSRC");
      if (   (newsrc = fopen(news_register,"r","mbc=40")) != 0
	  && (in_line = fgetl(newsrc)) != 0
	  && *in_line == '~') {
        fclose(newsrc); newsrc = 0;
	strcpy(news_register,"SYS$LOGIN:NEWSRC.LOCALDB");
	}
      }
    else {
      *cp-- = '\0';
      *(cp - 1) = '\0';
      strcpy(newsrcnode,in_line+1);
      lower_case(newsrcnode);
      if (nntp_sts == 2 || nntp_sts == 4) {
        if (strcmp(newsrcnode,nntp_node)) {
	  fclose(newsrc); newsrc = 0;
	  sprintf(news_register,"SYS$LOGIN:NEWSRC#%s",nntp_node);
	  while ((cp = strchr(news_register,'.')) != 0) *cp = '-';
	  if ((cp = strchr(news_register,'#')) != 0) *cp = '.';
          if (   (newsrc = fopen(news_register,"r","mbc=40")) != 0
	      && (in_line = fgetl(newsrc)) != 0
	      && *in_line == '~' && (cp = strchr(in_line+1,'~')) != 0) {
            *cp-- = '\0';
            *(cp - 1) = '\0';
            }
          else cp = 0;
          }
	}
      else strcpy(nntp_node,newsrcnode);

      if (cp && nntp_sts < 3) {
	nntp_proto = atoi(cp);
        if (!nntp_sts) nntp_sts = 4; else nntp_sts += 2;
	}
      nntp_client = 1;
      }
    }
  else if (newsrc && nntp_client) {
		/* newsrc file exists, but is not configured as a server
		   - AND NO LOGICALS FOR THE LOCAL DB EXIST */
    if (nntp_sts <= 0) {
      printf("NEWS: Logical names (NEWS_ROOT, NEWS_DEVICE) not defined\n");
      exit(1);
      }
    fclose(newsrc);
    newsrc = 0;
    sprintf(news_register,"SYS$LOGIN:NEWSRC#%s",nntp_node);
    while ((cp = strchr(news_register,'.')) != 0) *cp = '-';
    if ((cp = strchr(news_register,'#')) != 0) *cp = '.';
    if (   (nntp_sts < 3) && (newsrc = fopen(news_register,"r","mbc=40")) != 0
	&& (in_line = fgetl(newsrc)) != 0
	&& *in_line == '~' && (cp = strchr(in_line+1,'~')) != 0) {
      *cp-- = '\0';
      *(cp - 1) = '\0';
      }
    else cp = 0;
    }
  else if (newsrc && !in_line) {
    fclose(newsrc);
    newsrc = 0;
    }

#if FAST_LOAD
  if (newsrc) {
    fseek(newsrc,0,0);
    if ((in_line = fgetl(newsrc)) != 0)
      fast_loading = (strchr(in_line,'>') != 0);
    }
#endif

  if (nntp_client) {
    unsigned short len;
    $DESCRIPTOR(dsc,tmp);

    if (!*nntp_node) {
      if ((gotenv = getenv("NEWS_NNTP_SERVER")) != 0) strcpy(nntp_node,gotenv);
      else {
        len = 0;
        if (first_retr_call) get_input(&dsc,c$dsc("NEWS-SERVER Node: "),&len);
        if (!len) {
          printf("NEWS: No NNTP server node configured\n");
          exit(1);
          }
        tmp[len] = '\0';
        strcpy(nntp_node,tmp);
        }
      lower_case(nntp_node);
      }
    if (nntp_sts < 3) {
      if ((gotenv = getenv("NEWS_NNTP_PROTOCOL")) != 0) strcpy(tmp,gotenv);
      else {
        len = 0;
        if (first_retr_call) get_input_dflt(&dsc,c$dsc("NEWS-SERVER Protocol: "),&len,c$dsc("DECNET"),0);
        if (!len) strcpy(tmp,"DECNET");
        else tmp[len] = '\0';
	}
      lower_case(tmp);
      len = strlen(tmp);
      if (!*tmp) nntp_proto = 0;
      else if (!strncmp(tmp,"decnet",len)) nntp_proto= 0;
      else if (!strncmp(tmp,"cmutcp",len)) nntp_proto= 1;
      else if (!strncmp(tmp,"wintcp",len)) nntp_proto = 2;
      else if (!strncmp(tmp,"multinettcp",len)) nntp_proto = 3;
      else if (!strncmp(tmp,"ucxtcp",len)) nntp_proto = 4;
      else if (!strncmp(tmp,"exos",len)) nntp_proto = 5;
      else if (!strncmp(tmp,"tcpware",len)) nntp_proto = 6;
      else if (!strncmp(tmp,"tcp",len))
#if MULTINET
        nntp_proto = 3;
#elif TWG
        nntp_proto = 2;
#elif UCX
        nntp_proto = 4;
#elif EXOS
        nntp_proto = 5;
#elif TCPWARE
        nntp_proto = 6;
#else
        nntp_proto = 1;
#endif
      else {
        printf("NEWS: Not a valid NNTP transport protocol code\n");
        exit(1);
        }
      }
    }
  else {
    kid_valid = 0;
    if (init_lock()) exit(1);
    kid_valid = 1;
    itmfab = cc$rms_fab;
    itmfab.fab$b_bks = 3;
    itmfab.fab$b_fac = FAB$M_GET | FAB$M_PUT | FAB$M_UPD | FAB$M_DEL ;
    itmfab.fab$l_fna = ITM_FILENAME;
    itmfab.fab$b_fns = strlen(itmfab.fab$l_fna);
    itmfab.fab$l_fop = FAB$M_CIF;
    itmfab.fab$w_mrs = sizeof newsitm;
    itmfab.fab$b_org = FAB$C_IDX;
    itmfab.fab$b_rat = FAB$M_CR;
    itmfab.fab$b_rfm = FAB$C_FIX;
    itmfab.fab$b_shr = FAB$M_SHRDEL | FAB$M_SHRGET | FAB$M_SHRPUT | FAB$M_SHRUPD;
    itmfab.fab$l_xab = (char *) &xabkey_1;

    xabkey_1 = cc$rms_xabkey;
    xabkey_1.xab$b_dtp = XAB$C_BN8;
    xabkey_1.xab$b_flg = 0;
    xabkey_1.xab$w_pos0 = (char *) &newsitm.itm_num - (char *) &newsitm;
    xabkey_1.xab$b_ref = 0;
    xabkey_1.xab$b_siz0 = 8;
    xabkey_1.xab$l_nxt = (char *) &xabkey_2;

    xabkey_2 = cc$rms_xabkey;
    xabkey_2.xab$b_dtp = XAB$C_STG;
    xabkey_2.xab$b_flg = 0;
    xabkey_2.xab$w_pos0 = (char *) &newsitm.itm_id - (char *) &newsitm;
    xabkey_2.xab$w_pos1 = (char *) &newsitm.itm_grp - (char *) &newsitm;
    xabkey_2.xab$b_ref = 1;
    xabkey_2.xab$b_siz0 = IDLEN;
    xabkey_2.xab$b_siz1 = 4;
    xabkey_2.xab$l_nxt = (char *) &xabpro_1;

    xabpro_1 = cc$rms_xabpro;
    xabpro_1.xab$w_pro = 0xEE00;
    
    itmrab = cc$rms_rab;
    itmrab.rab$l_fab = &itmfab;
    itmrab.rab$b_krf = 0;
    itmrab.rab$b_ksz = 8;
    itmrab.rab$l_ubf = (char *) &newsitm;
    itmrab.rab$w_usz = sizeof newsitm;
    itmrab.rab$l_rbf = (char *) &newsitm;
    itmrab.rab$w_rsz = sizeof newsitm;

    sysprv();
    if (!((status = sys$open(&itmfab)) & 1)) {
#ifndef V58_UPGRADE_MANUAL
      if (!no_priv() && v58_upgrade()) status = sys$create(&itmfab);
      if (!(status & 1)) _c$cks(status);
#else
      puts("You need to perform an upgrade of the group and item files so that");
      puts("they can be read by this version of NEWS");
      _c$cks(status);
#endif
      }
    if (status == RMS$_CREATED) {
      printf("\tNEWS - first time installation...\n");
      first_time();
      printf("\tNEWS - create new ITEM file\n");
      }
    _c$cks(sys$connect(&itmrab));
    if (sys$get(&itmrab) & 1) {
      if (itmrab.rab$w_rsz < sizeof newsitm) {
        v59_file = 1;
#ifndef V60_UPGRADE_MANUAL
        if (!no_priv()) {
          sys$close(&itmfab);
          v60_upgrade();
          _c$cks(sys$open(&itmfab));
          _c$cks(sys$connect(&itmrab));
          v59_file = 0;
          }
#endif
        }
      }      
    nosysprv();

    /* open the index newsgroup descriptor file for full access */
    grpfab = cc$rms_fab;
    grpfab.fab$b_bks = 3;
    grpfab.fab$b_fac = FAB$M_GET | FAB$M_PUT | FAB$M_UPD | FAB$M_DEL ;
    grpfab.fab$l_fna = GRP_FILENAME;
    grpfab.fab$b_fns = strlen(grpfab.fab$l_fna);
    grpfab.fab$l_fop = FAB$M_CIF;
    grpfab.fab$w_mrs = sizeof newsgrp;
    grpfab.fab$b_org = FAB$C_IDX;
    grpfab.fab$b_rat = FAB$M_CR;
    grpfab.fab$b_rfm = FAB$C_FIX;
    grpfab.fab$b_shr = FAB$M_SHRDEL | FAB$M_SHRGET | FAB$M_SHRPUT | FAB$M_SHRUPD;
    grpfab.fab$l_xab = (char *) &xabkey_3;

    xabkey_3 = cc$rms_xabkey;
    xabkey_3.xab$b_dtp = XAB$C_STG;
    xabkey_3.xab$b_flg = 0;
    xabkey_3.xab$w_pos0 = (char *) &newsgrp.grp_name - (char *) &newsgrp;
    xabkey_3.xab$b_ref = 0;
    xabkey_3.xab$b_siz0 = SUBJLEN;
    xabkey_3.xab$l_nxt = (char *) &xabkey_4;

    xabkey_4 = cc$rms_xabkey;
    xabkey_4.xab$b_dtp = XAB$C_BN4;
    xabkey_4.xab$b_flg = 0;
    xabkey_4.xab$w_pos0 = (char *) &newsgrp.grp_num - (char *) &newsgrp;
    xabkey_4.xab$b_ref = 1;
    xabkey_4.xab$b_siz0 = 4;
    xabkey_4.xab$l_nxt = (char *) &xabpro_2;

    xabpro_2 = cc$rms_xabpro;
    xabpro_2.xab$w_pro = 0xEE00;

    grprab = cc$rms_rab;
    grprab.rab$l_fab = &grpfab;
    grprab.rab$l_ubf = (char *) &newsgrp;
    grprab.rab$w_usz = sizeof newsgrp;
    grprab.rab$l_rbf = (char *) &newsgrp;
    grprab.rab$w_rsz = sizeof newsgrp;

    sysprv();
    if (no_priv()) status = sys$open(&grpfab);
    else status = sys$create(&grpfab);
    if (!(status & 1)) _c$cks(status);
    if (status == RMS$_CREATED) {
      _c$cks(sys$connect(&grprab));
      printf("\tNEWS - create new GROUP file\n");
      strcpy(newsgrp.grp_name,"NEWSGROUP INDEX");
      newsgrp.grp_num = 0;
      newsgrp.grp_topnum = 0;
      newsgrp.grp_count = 0;
      newsgrp.grp_life = GRP_TIME;
      newsgrp.grp_itmlife = EXP_TIME;
      time((time_t *) &newsgrp.grp_credate);
      newsgrp.grp_entdate = newsgrp.grp_credate;
      newsgrp.grp_reg = newsgrp.grp_unread = 0;
      newsgrp.grp_ia = (ITM_PTR) 0;
      newsgrp.grp_iasize = newsgrp.grp_iavd = 0;
      newsgrp.grp_iavdsize = newsgrp.grp_iapaste = newsgrp.grp_c_itm = 0;
      newsgrp.grp_display_indx = newsgrp.grp_flags = 0;
      newsgrp.grp_reg_text = (char *) 0;
      *(newsgrp.grp_srvnode) = '\0';
      newsgrp.grp_flags |= NEWS_M_NNTPCACHE;
      newsgrp.grp_srvcache = 2;

      grprab.rab$w_rsz = sizeof newsgrp;
      _c$cks(sys$put(&grprab));
      }
    else _c$cks(sys$connect(&grprab));

    nosysprv();
    }
#if FAST_LOAD
  if (fast_loading && !all_loaded) read_reg_file();
  else
#endif
  open_groups(DIR_ALL);
}

/*
 *  open_groups
 * 
 *  Set up the newsgroup array
 */

#if NNTP_CLIENT

static int initng_count = 0,
	   initng_sorted = 0;	/* True => ga[] is sorted by group name */

static int initng(cp)
  char *cp;
{
  register int g;
  int first, last, retval = 0;
  char newsg[256],mod[132];

  if (first_retr_call && !(++initng_count % 10)
#if FAST_LOAD
                      && (!fast_loading)
#endif
						) printf(".");
  chop_str(cp,'\n');
  if (sscanf(cp,"%s %d %d %s",newsg,&last,&first,mod) == 4) {
    ga[0]->grp_count += 1;
    newsg[SUBJLEN-1] = '\0';
    strcpy(newsgrp.grp_name,newsg);
    ++newsgrp.grp_num;
    newsgrp.grp_topnum = last;
    newsgrp.grp_count = max(((last - first) + 1),0);
    if (!last) newsgrp.grp_count = 0;
    newsgrp.grp_topic[0] = '\0';
    newsgrp.grp_notice[0] = '\0';
    newsgrp.grp_unread = newsgrp.grp_count;
    newsgrp.grp_firstnum = first;
    newsgrp.grp_flags = NEWS_M_NNTPSRV;
    if (tolower(*mod) == 'm') newsgrp.grp_flags |= NEWS_M_MAILMODERATE;
    if (tolower(*mod) != 'n') newsgrp.grp_flags |= NEWS_M_WRITE_ACCESS;
    else newsgrp.grp_flags |= NEWS_M_NOWRITE_SET;
    newsgrp.grp_flags |= NEWS_M_ACCESS_CHECKED;
    if (ga_size > (ga_malloc - 5)) {
      ga_malloc += 250;
      ga = realloc(ga,ga_malloc * (sizeof *ga));
      }
    /* figure out where to add the group */
    g = ++ga_size;	/* default is at end of [expanded] list */
    if (initng_sorted) {
      while (--g	/* while greater than 0 and name out of order... */
	  && (*newsg < *ga[g]->grp_name || strcmp(newsg,ga[g]->grp_name) < 0))
	continue;
      ++g;		/* insert new group here */
      if (g < ga_size)	/* shift rest of array up one slot */
	memmove(&ga[g+1], &ga[g], (ga_size - g) * (sizeof *ga));
    }
    /* add the group */
    ga[g] = malloc(sizeof newsgrp);
    *ga[g] = newsgrp;
    ga[g]->grp_display_indx = ++grp_display_size;
    retval = g;
    if (initng_sorted) {
      int gi = 0;
      for (g = 1; g <= ga_size; ++g)
	if (ga[g]->grp_display_indx)
	  ga[g]->grp_display_indx = ++gi;
      }
    }
  return(retval);
}

static int initng_compare(v_left, v_right)
  const void *v_left, *v_right;
{
  const GRP_PTR *left = v_left, *right = v_right;
  return strcmp((*left)->grp_name, (*right)->grp_name);
}

static void initng_sort()
{
  if (!initng_sorted) {
    register int g, i = 0;
    /* sort the unordered group array */
    qsort(ga+1, ga_size, sizeof *ga, initng_compare);
    for (g = 1; g <= ga_size; ++g)
      if (ga[g]->grp_display_indx)
	ga[g]->grp_display_indx = ++i;
    initng_sorted = 1;
    }
}

static int inittopics(cp)
  char *cp;
{
  char *tp;
  int g;

  chop_str(cp,'\n');
  tp = cp;
  while (*tp && !isspace(*tp)) ++tp;
  if (*tp) *tp++ = '\0';
  lower_case(cp);
  if ((g = ga_exact_name(cp)) != 0) ga[g]->grp_topic[0] = '\0';
  if (!*tp || !g) return 0;
  while (isspace(*tp)) ++tp;
  strncpy(ga[g]->grp_topic,tp,124);
  ga[g]->grp_topic[124] = '\0';
  return 0;
}
  
#if NNTP_USE_XGNOTICE
static int initnotices(cp)
  char *cp;
{
  char *tp;
  int g;

  chop_str(cp,'\n');
  if ((tp = chop_str_plus(cp,' ')) == 0) return 0;
  lower_case(cp);
  if ((g = ga_exact_name(cp)) != 0) {
    strncpy(ga[g]->grp_notice,tp,124);
    ga[g]->grp_notice[124] = '\0';
    }
  return 0;
}
#endif

int initgroups(g)
  int g;
{
  int c,f,l;
  char *rcall;

  sprintf(err_oline,"GROUP %s",ga[g]->grp_name);
  if (nntp_one_call(nntp_node,nntp_proto,err_oline,&rcall) == 211) {
    sscanf(rcall,"211 %d %d %d %*s",&c,&f,&l);
    ga[g]->grp_count = c;
    ga[g]->grp_firstnum = f;
    ga[g]->grp_topnum = l;
    }
  return 0;
}
#endif

void open_groups(cd)
  int cd;
{
  char *cp, *ap, *op, *dp,
       mod[132], acline[256], cknode[132], dir_name[256];
  FILE *fpd;
  struct FAB dirfab;

#if NNTP_CLIENT
  if (nntp_client) {
    initng_count = 0;
    if (first_retr_call) {
      switch (nntp_proto) {
 	case 1:  cp = "TCP [CMU]";  break;
 	case 2:  cp = "TCP [WIN]";  break;
 	case 3:  cp = "TCP [MultiNet]";  break;
 	case 4:  cp = "TCP [UCX]";  break;
 	case 5:  cp = "TCP [EXOS]";  break;
 	case 6:  cp = "TCP [TCPware]";  break;
 	default: cp = "DECnet";  break;
 	}
      printf("Connecting to server @ %s (using %s)\n",nntp_node,cp);
      }

#if FAST_LOAD
    if (fast_loading && !all_loaded) start_group_load();
    else
#endif
    {
    ga_malloc = 50;
    ga = (GRP_PTR *) malloc(ga_malloc * (sizeof *ga));

    cur_dir_type = DIR_ALL;

    newsgrp.grp_num = 0;
    newsgrp.grp_topnum = 0;
    newsgrp.grp_count = 0;
    newsgrp.grp_life = GRP_TIME;
    newsgrp.grp_itmlife = EXP_TIME;
    time((time_t *)&newsgrp.grp_credate);
    newsgrp.grp_entdate = newsgrp.grp_credate;
    newsgrp.grp_reg = newsgrp.grp_unread = 0;
    newsgrp.grp_ia = (ITM_PTR)0;
    newsgrp.grp_iasize = newsgrp.grp_iavd = 0;
    newsgrp.grp_display_indx = newsgrp.grp_iavdsize = newsgrp.grp_iapaste = 0;
    newsgrp.grp_flags = newsgrp.grp_c_itm = 0;
    newsgrp.grp_reg_text = (char *)0;
    strcpy(newsgrp.grp_srvnode,nntp_node);
    newsgrp.grp_srvproto = nntp_proto;
    newsgrp.grp_flags = NEWS_M_NNTPSRV;
    newsgrp.grp_srvcache = 0;
    *newsgrp.grp_topic = '\0';
    *newsgrp.grp_notice = '\0';

    ga[0] = malloc(sizeof newsgrp);
    *ga[0] = newsgrp;
    newsgrp.grp_reg = NEWGROUP_REG | (1 & (profile_flags & PROFILE_NEWREGISTER));
    ga_size = 0;
    grp_display_size = 0;
    }

    if (first_retr_call) printf("Retrieving newsgroups:");
    nntp_get_info(nntp_node,nntp_proto,"LIST",215,0,initng);
    initng_sort();
   if (first_retr_call) printf(":");	/* end of LIST load */
#if NNTP_USE_XGTITLE
    if (!nntp_get_info(nntp_node,nntp_proto,"XGTITLE *",282,0,inittopics))
#endif
    nntp_get_info(nntp_node,nntp_proto,"LIST NEWSGROUPS",215,0,inittopics);
#if NNTP_USE_XGNOTICE
    nntp_get_info(nntp_node,nntp_proto,"XGNOTICE *",282,0,initnotices);
#endif
  
/*#### TOO SLOW !!  
    {
      register int g, i = 0;
      for (g = 1; g <= ga_size; ++g)
	if (ga[g]->grp_firstnum <= 1) {
	  if (first_retr_call && !i) printf("\nRetrieving newsgroup counts:");
	  if (first_retr_call && !(++i % 10)) printf(".");
	  initgroups(g);
	  }
    }
  ####*/
  
    if (first_retr_call) printf("\n");

    if (!ga[0]->grp_count) {
      printf("NEWS: Failed to retrieve newsgroups from remote node\n");
      printf("      Connection terminated\n");
      exit(1);
      }
    first_retr_call = 0;
    }
#endif

#if !NNTP_CLIENT_ONLY
#if NNTP_CLIENT
  else {
#endif
#if FAST_LOAD
    if (fast_loading && !all_loaded) start_group_load();
    else
#endif
    {
    grprab.rab$l_kbf = (char *) c$rfi(0);
    grprab.rab$b_ksz = 4;
    grprab.rab$b_krf = 1;
    grprab.rab$l_rop = RAB$M_RRL | RAB$M_NLK;
    grprab.rab$b_rac = RAB$C_KEY;
    _c$cks(sys$get(&grprab));

    ga_malloc = (newsgrp.grp_topnum + 5);
    ga = (GRP_PTR *) malloc(ga_malloc * (sizeof *ga));

    cur_dir_type = DIR_ALL;
    ga[0] = (GRP_PTR) malloc(sizeof newsgrp);
    *ga[0] = newsgrp;
    ga_size = 0;
    grp_display_size = 0;
    }

    dirfab = cc$rms_fab;
    dirfab.fab$b_fac = FAB$M_GET;
    dirfab.fab$l_fna = dir_name;
    dirfab.fab$b_shr = FAB$M_SHRGET | FAB$M_SHRPUT | FAB$M_SHRUPD;

    grprab.rab$b_krf = 0;
    grprab.rab$b_rac = RAB$C_SEQ;
    sys$rewind(&grprab);
    while ((status = sys$get(&grprab)) & 1) {

      if (!newsgrp.grp_num) continue;
#if FAST_LOAD
      if (fast_loading && !all_loaded) {
        newsgrp.grp_reg = 0;  /* GBW for unregisted groups */
        newsgrp.grp_unread = newsgrp.grp_count;
        newsgrp.grp_ia = 0;
        newsgrp.grp_iasize = 0;
        newsgrp.grp_iavd = 0;
        newsgrp.grp_iavdsize = 0;
        newsgrp.grp_iapaste = 0;
        newsgrp.grp_c_itm = 0;
        newsgrp.grp_display_indx = 0;
        newsgrp.grp_reg_text = 0;
        newsgrp.grp_topreadnum = 0;
                          /* ensure no group gets 'funny' info about them */
      }
#endif

#if FULL_DIRECTORY_ACCESS_CHECK

      sprintf(dir_name,Dir_template,util_dir(newsgrp.grp_name));
      dp = strrchr(dir_name,'.'); *dp = ']';
      dp = strchr(dir_name,'^'); *dp = '.';
      dirfab.fab$b_fns = strlen(dirfab.fab$l_fna);
      if (sys$open(&dirfab) & 1) sys$close(&dirfab);
      else continue;
#endif

      newsgrp.grp_flags &= ~(NEWS_M_WRITE_ACCESS | NEWS_M_MOD_ACCESS | NEWS_M_MOD_USER | NEWS_M_ACCESS_CHECKED);
      if (no_priv() && newsgrp.grp_flags & NEWS_M_RESTRICT_SET) {
        if (newsgrp.grp_flags & NEWS_M_MAILMODERATE) {
          sprintf(mod,"%s@%s",usr_username,Node_address);
          if (!strcmp(mod,moderator_address(newsgrp.grp_name))) {
            newsgrp.grp_flags |= (NEWS_M_ACCESS_CHECKED | NEWS_M_MOD_USER | NEWS_M_WRITE_ACCESS);
            }
          }
        if (!(newsgrp.grp_flags & NEWS_M_ACCESS_CHECKED)) {

#if DIRECTORY_ACCESS_CHECK && (!FULL_DIRECTORY_ACCESS_CHECK)
          sprintf(dir_name,Dir_template,util_dir(newsgrp.grp_name));
          dp = strrchr(dir_name,'.'); *dp = ']';
          dp = strchr(dir_name,'^'); *dp = '.';
          dirfab.fab$b_fns = strlen(dirfab.fab$l_fna);
          if (sys$open(&dirfab) & 1) sys$close(&dirfab);
          else continue;
#endif

          newsgrp.grp_flags |= NEWS_M_ACCESS_CHECKED;
          sprintf(itm_fname,Access_template,util_dir(newsgrp.grp_name));
          sysprv();
          if (!(fpd = fopen(itm_fname,"r"))) {
            nosysprv();

#if (!DIRECTORY_ACCESS_CHECK) && (!FULL_DIRECTORY_ACCESS_CHECK)
            sprintf(dir_name,Dir_template,util_dir(newsgrp.grp_name));
            dp = strrchr(dir_name,'.'); *dp = ']';
            dp = strchr(dir_name,'^'); *dp = '.';
            dirfab.fab$b_fns = strlen(dirfab.fab$l_fna);
            if (sys$open(&dirfab) & 1) sys$close(&dirfab);
            else continue;
#endif
            }
          else {
            int non_member = 1;

            if (!(newsgrp.grp_flags & NEWS_M_NOWRITE_SET))  newsgrp.grp_flags |= NEWS_M_WRITE_ACCESS;
            while (fgets(acline,256,fpd)) {
              if (*acline == '#') continue;
              if ((cp = strchr(acline,'\n')) != 0) *cp = ' ';
              lower_case(acline);
              op = chop_str(acline,':');
              cp = chop_str(acline,' ');
              ap = chop_str(acline,'@');
              if (!strcmp(acline,usr_username) || wild_match(usr_username,acline) || idmatch(acline)) {
                if (cp) *cp = ' ';
                if (ap) *ap = '@';
                if (strchr(acline,'@')) {
                  sprintf(cknode,"@%s ",news_node);
                  if (!substrcmp(acline,cknode)) continue;
                  }
                if (substrcmp(cp," write ")) newsgrp.grp_flags |= NEWS_M_WRITE_ACCESS;
                if (substrcmp(cp," nowrite ")) newsgrp.grp_flags &= ~NEWS_M_WRITE_ACCESS;
                if (   substrcmp(cp," moderate ")
                    || substrcmp(cp," moderator ")
                    || substrcmp(cp," supervise "))
		  newsgrp.grp_flags |= (NEWS_M_MOD_USER | NEWS_M_WRITE_ACCESS);
                non_member = 0;
                }
              }
            fclose(fpd);
            nosysprv();
            if (non_member) continue;
            }
          }
        }

    ga[++ga_size] = (GRP_PTR) malloc(sizeof newsgrp);
    newsgrp.grp_unread = newsgrp.grp_count;
    newsgrp.grp_reg = NEWGROUP_REG | (1 & (profile_flags & PROFILE_NEWREGISTER));
    newsgrp.grp_ia = (ITM_PTR) 0;
    newsgrp.grp_iasize = 0;
      newsgrp.grp_iavd = newsgrp.grp_iavdsize = newsgrp.grp_iapaste = 0;
    newsgrp.grp_c_itm = 0;
    newsgrp.grp_reg_text = (char *) 0;
      *ga[ga_size] = newsgrp;
      ga[ga_size]->grp_display_indx = ++grp_display_size;
      }
    if (status != RMS$_EOF) c$cks(status);
#if NNTP_CLIENT
    }
#endif
#endif
  read_reg_file();
}

/*
 *  closefiles
 *
 *  Close rms files, write the register file, shutdown the screen, and
 *  send any printer output to a nominated queue.
 */

void
closefiles()
{
#if !NNTP_CLIENT
  if (!nntp_client) {
    sysprv();
    sys$close(&itmfab);
    sys$close(&grpfab);
    nosysprv();
    close_hist_file();
    }
#endif
  close_mail_file();
  close_nntp_file();
  write_reg_file();
  noscreen();
  print_exit();
  log_to_usage_file(1,0);
}

/*
 *  get_input
 *
 *  much the same as lib$get_input - but uses SMG display
 */

int get_input(get_str,prompt_str,out_len)
  struct dsc$descriptor_s *get_str,*prompt_str;
  unsigned short *out_len;
{
  return(get_input_dflt(get_str,prompt_str,out_len,0,0));
}

/*
 *  get_input_dflt
 *
 *  get input, with default input string supplied
 */

int get_input_dflt(get_str,prompt_str,out_len,inistr,trm)
  struct dsc$descriptor_s *get_str,*prompt_str,*inistr;
  unsigned short *out_len,*trm;
{
  int status;

  if (smg_active) {
    smg$erase_display(&trailer_vd,c$ac(2),c$ac(1),c$ac(2),c$ac(devcol));
    smg$set_cursor_abs(&trailer_vd,c$ac(2),c$ac(1));
    smg$end_pasteboard_update(&pid);
    edit_m_on();
    }
  status = smg$read_composed_line(&kid,&keytab,get_str,prompt_str,out_len,((smg_active) ? &trailer_vd : 0),0,inistr,0,0,0,trm);
  if (((status == SS$_ABORT) || (status == SS$_CANCEL)) && news_lock_alarm) {
    closefiles();
    exit(1);
    }
  if (smg_active) {
    edit_m_off();
    smg$begin_pasteboard_update(&pid);
    }
  return((status == SMG$_EOF) ? RMS$_EOF : status);
}

#if FAST_LOAD
void start_group_load()
{
  if (nntp_client) {
    ga_malloc = 50;
    ga = (GRP_PTR *) malloc(ga_malloc * (sizeof *ga));

    cur_dir_type = DIR_ALL;

    newsgrp.grp_num = 0;
    newsgrp.grp_topnum = 0;
    newsgrp.grp_count = 0;
    newsgrp.grp_life = GRP_TIME;
    newsgrp.grp_itmlife = EXP_TIME;
    time((time_t *)&newsgrp.grp_credate);
    newsgrp.grp_entdate = newsgrp.grp_credate;
    newsgrp.grp_iasize = newsgrp.grp_unread = 0;
    newsgrp.grp_ia = 0;
    newsgrp.grp_reg = newsgrp.grp_iavd = 0;
    newsgrp.grp_display_indx = newsgrp.grp_iavdsize = newsgrp.grp_iapaste = 0;
    newsgrp.grp_flags = newsgrp.grp_c_itm = 0;
    newsgrp.grp_reg_text = (char *)0;
    strcpy(newsgrp.grp_srvnode,nntp_node);
    newsgrp.grp_srvproto = nntp_proto;
    newsgrp.grp_flags = NEWS_M_NNTPSRV;
    newsgrp.grp_srvcache = 0;
    *newsgrp.grp_topic = '\0';
    *newsgrp.grp_notice = '\0';

    ga[0] = malloc(sizeof newsgrp);
    *ga[0] = newsgrp;
    newsgrp.grp_reg = NEWGROUP_REG | (1 & (profile_flags & PROFILE_NEWREGISTER));
    ga_size = 0;
    grp_display_size = 0;
    }
  else {
    grprab.rab$l_kbf = (char *) c$rfi(0);
    grprab.rab$b_ksz = 4;
    grprab.rab$b_krf = 1;
    grprab.rab$l_rop = RAB$M_RRL | RAB$M_NLK;
    grprab.rab$b_rac = RAB$C_KEY;
    _c$cks(sys$get(&grprab));
    ga_malloc = (fast_loading ? 20 : (newsgrp.grp_topnum + 5));
    ga = (GRP_PTR *) malloc(ga_malloc * (sizeof *ga));
    cur_dir_type = DIR_ALL;
    ga[0] = (GRP_PTR) malloc(sizeof newsgrp);
    *ga[0] = newsgrp;
    ga[0]->grp_display_indx = 0;
    ga_size = 0;
    grp_display_size = 0;
    }
}

/*
 * 
 *
 * 
 */

int read_group( group_name )
  char *group_name;
{
  if (nntp_client) {
    char *buffer;
    int first, last, count;

    sprintf(err_oline,"GROUP %s",group_name);
    if (   (nntp_one_call(nntp_node,nntp_proto,err_oline,&buffer) == 211)
	&& (sscanf(buffer,"211 %d %d %d",&count, &first, &last) == 3)) {
      sprintf(err_oline,"%s %d %d y",group_name,last,first);
      return(initng(err_oline));
      }
    sprintf(err_oline,"Group %s -> %s",group_name,buffer);
    err_line(err_oline);
    return(0);
    }
  else {
    int r_status, ga_indx;
    char *cp, *ap, *op, *dp,
         mod[132], acline[256], cknode[132], dir_name[256];
    FILE *fpd;
    struct FAB dirfab;

    dirfab = cc$rms_fab;
    dirfab.fab$b_fac = FAB$M_GET;
    dirfab.fab$l_fna = dir_name;
    dirfab.fab$b_shr = FAB$M_SHRGET | FAB$M_SHRPUT | FAB$M_SHRUPD;

    grprab.rab$l_kbf = group_name;
    grprab.rab$b_ksz = strlen(group_name);
    grprab.rab$b_krf = 0;
    grprab.rab$l_rop = RAB$M_RRL | RAB$M_NLK;
    grprab.rab$b_rac = RAB$C_KEY;
    r_status = sys$get(&grprab);
    if (r_status == RMS$_RNF) return(0) ;
    if ( r_status != RMS$_EOF ) c$cks(r_status);
    if (!newsgrp.grp_num) return(0);

#if FULL_DIRECTORY_ACCESS_CHECK
    sprintf(dir_name,Dir_template,util_dir(newsgrp.grp_name));
    dp = strrchr(dir_name,'.'); *dp = ']';
    dp = strchr(dir_name,'^'); *dp = '.';
    dirfab.fab$b_fns = strlen(dirfab.fab$l_fna);
    if (sys$open(&dirfab) & 1) sys$close(&dirfab);
    else return(0);
#endif

    newsgrp.grp_flags &= ~(NEWS_M_WRITE_ACCESS | NEWS_M_MOD_ACCESS | NEWS_M_MOD_USER | NEWS_M_ACCESS_CHECKED);
    if (no_priv() && newsgrp.grp_flags & NEWS_M_RESTRICT_SET) {
      if (newsgrp.grp_flags & NEWS_M_MAILMODERATE) {
        sprintf(mod,"%s@%s",usr_username,Node_address);
        if (!strcmp(mod,moderator_address(newsgrp.grp_name))) {
          newsgrp.grp_flags |= (NEWS_M_ACCESS_CHECKED | NEWS_M_MOD_USER | NEWS_M_WRITE_ACCESS);
          }
        }
      if (!(newsgrp.grp_flags & NEWS_M_ACCESS_CHECKED)) {

#if DIRECTORY_ACCESS_CHECK && (!FULL_DIRECTORY_ACCESS_CHECK)
        sprintf(dir_name,Dir_template,util_dir(newsgrp.grp_name));
        dp = strrchr(dir_name,'.'); *dp = ']';
        dp = strchr(dir_name,'^'); *dp = '.';
        dirfab.fab$b_fns = strlen(dirfab.fab$l_fna);
        if (sys$open(&dirfab) & 1) sys$close(&dirfab);
        else return(0);
#endif

        newsgrp.grp_flags |= NEWS_M_ACCESS_CHECKED;
        sprintf(itm_fname,Access_template,util_dir(newsgrp.grp_name));
        sysprv();
        if (!(fpd = fopen(itm_fname,"r"))) {
          nosysprv();

#if (!DIRECTORY_ACCESS_CHECK) && (!FULL_DIRECTORY_ACCESS_CHECK)
          sprintf(dir_name,Dir_template,util_dir(newsgrp.grp_name));
          dp = strrchr(dir_name,'.'); *dp = ']';
          dp = strchr(dir_name,'^'); *dp = '.';
          dirfab.fab$b_fns = strlen(dirfab.fab$l_fna);
          if (sys$open(&dirfab) & 1) sys$close(&dirfab);
          else return(0);
#endif
          }
        else {
          int non_member = 1;

          if (!(newsgrp.grp_flags & NEWS_M_NOWRITE_SET))  newsgrp.grp_flags |= NEWS_M_WRITE_ACCESS;
          while (fgets(acline,256,fpd)) {
            if (*acline == '#') continue;
            if ((cp = strchr(acline,'\n')) != 0) *cp = ' ';
            lower_case(acline);
            cp = chop_str(acline,' ');
            ap = chop_str(acline,'@');
            op = chop_str(acline,':');
            if (!strcmp(acline,usr_username) || wild_match(usr_username,acline) || idmatch(acline)) {
              if (cp) *cp = ' ';
              if (ap) *ap = '@';
              if (strchr(acline,'@')) {
                sprintf(cknode,"@%s ",news_node);
                if (!substrcmp(cp,cknode)) continue;
                }
              if (substrcmp(cp," write ")) newsgrp.grp_flags |= NEWS_M_WRITE_ACCESS;
              if (substrcmp(cp," nowrite ")) newsgrp.grp_flags &= ~NEWS_M_WRITE_ACCESS;
              if (   substrcmp(cp," moderate ")
                  || substrcmp(cp," moderator ")
                  || substrcmp(cp," supervise "))
	        newsgrp.grp_flags |= (NEWS_M_MOD_USER | NEWS_M_WRITE_ACCESS);
              non_member = 0;
              }
            }
          fclose(fpd);
          nosysprv();
          if (non_member) return(0);
          }
        }
      }
    if (ga_size > (ga_malloc - 5)) {
      ga_malloc += 50;
      ga = realloc(ga,ga_malloc * (sizeof *ga));
      }
    ga_indx = ++ga_size;
    for (; 
         (ga_indx>1) && (0 > strcmp(newsgrp.grp_name, ga[ga_indx-1]->grp_name));
	 --ga_indx)
      ga[ga_indx] = ga[ga_indx-1];
    ga[ga_indx] = (GRP_PTR) malloc(sizeof newsgrp);
    newsgrp.grp_unread = newsgrp.grp_count;
    newsgrp.grp_reg = NEWGROUP_REG | (1 & (profile_flags & PROFILE_NEWREGISTER));
    newsgrp.grp_ia = (ITM_PTR) 0;
    newsgrp.grp_iasize = 0;
    newsgrp.grp_iavd = newsgrp.grp_iavdsize = newsgrp.grp_iapaste = 0;
    newsgrp.grp_c_itm = 0;
    newsgrp.grp_reg_text = (char *) 0;
    *ga[ga_indx] = newsgrp;
    ga[ga_indx]->grp_display_indx = ++grp_display_size;
    return( ga_indx );
    }
}
#endif
