/*
**++
**  FACILITY:
**      NEWSMODE
**
**  ABSTRACT:
**      Map up and down arrow keys into NEWS, with an exit handler to restore
**      key settings
**
**  AUTHOR:
**      Geoff Huston
**
**  COPYRIGHT:
**      Copyright  1988,1989,1990
**--
**/

#if vax11c
#module NEWSMODE "V6.0"
#endif

#define _NEWSMODE_C

#include "newsinclude.h"
#include "newsextern.h"

#if vax11c
#include iodef
#include ttdef
#include tt2def
#else
#include <iodef.h>
#include <ttdef.h>
#include <tt2def.h>
#endif
static unsigned short   chan = 0, iosb[4];
static int  old_char_buf[3],
       new_char_buf[3] = {0,0,0},                        
       edit_on_buf[3],
       brd_char_buf[3],
       exit_reason = 0,
       b_trap_flag = 1,
       b_trap_state = 0,
       line_flag = 1;
static struct { int reserved, (*handler) __ARGS((int *)), argc, *arg1; }
       exit_handler_block = {0,exit_mode,1,&exit_reason};

extern int sys$assign(), sys$dassgn(), sys$qiow(), sys$cancel(),
	   sys$dclexh(), sys$canexh();

/*
 *  exit_mode
 *
 *  Exit handler, to reset arrow key definition
 */

static int exit_mode(reason)
    int *reason;
{
    int i;
    sys$cancel(chan);
    sys$qiow(0,chan,IO$_SETMODE,iosb,0,0,
	     old_char_buf,sizeof old_char_buf,0,0,0,0);
    for (i = 0 ; i < 3 ; ++i) new_char_buf[i] = old_char_buf[i];
    sys$dassgn(chan);
    chan = 0;
    return *reason;
}

unsigned int page_size()
{
    unsigned short  *w;
    unsigned char page[12];

    if (devrow && devcol) return 0;
    sys$assign(c$dsc("SYS$INPUT"),&chan,0,0);
    sys$qiow(0,chan,IO$_SENSEMODE,iosb,0,0,
	     page,sizeof page,0,0,0,0);
    sys$dassgn(chan);
    w = (unsigned short *) &page[2];
    if (!devcol) devcol = (*w ? *w : 80);
    if (!devrow) devrow = (page[7] ? page[7]: 24);
    return 1;
}

/*
 *  chg_mode
 *
 *  Turn off line edit mode to allow arrow keys to be sent to application
 */

void chg_mode()
{
    int i;
    if (!chan) sys$assign(c$dsc("SYS$INPUT"),&chan,0,0);
    sys$qiow(0,chan,IO$_SENSEMODE,iosb,0,0,
	     old_char_buf,sizeof old_char_buf,0,0,0,0);
    sys$dclexh(&exit_handler_block);
    for (i = 0 ; i < 3 ; ++i)
      edit_on_buf[i] = new_char_buf[i] = old_char_buf[i];
    if (line_flag) new_char_buf[2] &= ~(TT2$M_EDITING);
    edit_on_buf[2] |= TT2$M_EDITING ;
    edit_m_off();
}

/*
 *  reset_mode
 *
 *  Revert mode to original, and cancel exit handler
 */

void
reset_mode()
{
  sys$canexh(&exit_handler_block);
  line_state = 0;
  if (line_flag) {
    smg$delete_key_def(&keytab,c$dsc("UP"),0);
    smg$delete_key_def(&keytab,c$dsc("DOWN"),0);
    smg$delete_key_def(&keytab,c$dsc("UP"),c$dsc("GOLD"));
    smg$delete_key_def(&keytab,c$dsc("DOWN"),c$dsc("GOLD"));
    }
  exit_reason = 1;
  exit_mode(&exit_reason);
}

/*
 *  edit_m_on
 *  edit_m_off
 *
 *  Turn on/off line edit mode
 */

void edit_m_off()
{
  line_state = 1;
  if (line_flag) {
    sys$qiow(0,chan,IO$_SETMODE,iosb,0,0,
	     new_char_buf,sizeof new_char_buf,0,0,0,0);
    smg$add_key_def(&keytab,c$dsc("UP"),0,c$ac(SMG$M_KEY_TERMINATE+SMG$M_KEY_NOECHO),c$dsc("UP"),0);
    smg$add_key_def(&keytab,c$dsc("DOWN"),0,c$ac(SMG$M_KEY_TERMINATE+SMG$M_KEY_NOECHO),c$dsc("DOWN"),0);
    smg$add_key_def(&keytab,c$dsc("UP"),c$dsc("GOLD"),c$ac(SMG$M_KEY_TERMINATE+SMG$M_KEY_NOECHO),c$dsc("TOP"),0);
    smg$add_key_def(&keytab,c$dsc("DOWN"),c$dsc("GOLD"),c$ac(SMG$M_KEY_TERMINATE+SMG$M_KEY_NOECHO),c$dsc("BOTTOM"),0);
    smg$add_key_def(&keytab,c$dsc("PREV_SCREEN"),0,c$ac(SMG$M_KEY_TERMINATE+SMG$M_KEY_NOECHO),c$dsc("SCUP"),0);
    smg$add_key_def(&keytab,c$dsc("PREV_SCREEN"),c$dsc("GOLD"),c$ac(SMG$M_KEY_TERMINATE+SMG$M_KEY_NOECHO),c$dsc("TOP"),0);
    smg$add_key_def(&keytab,c$dsc("NEXT_SCREEN"),0,c$ac(SMG$M_KEY_TERMINATE+SMG$M_KEY_NOECHO),c$dsc("SCDOWN"),0);
    smg$add_key_def(&keytab,c$dsc("NEXT_SCREEN"),c$dsc("GOLD"),c$ac(SMG$M_KEY_TERMINATE+SMG$M_KEY_NOECHO),c$dsc("BOTTOM"),0);
    }
}

void edit_m_on()
{
  line_state = 0;
  if (line_flag) {
    smg$delete_key_def(&keytab,c$dsc("UP"),0);
    smg$delete_key_def(&keytab,c$dsc("DOWN"),0);
    smg$delete_key_def(&keytab,c$dsc("UP"),c$dsc("GOLD"));
    smg$delete_key_def(&keytab,c$dsc("DOWN"),c$dsc("GOLD"));
    sys$qiow(0,chan,IO$_SETMODE,iosb,0,0,
	     edit_on_buf,sizeof edit_on_buf,0,0,0,0);
    }
}

int set_line()
{
  char sw[20];
  unsigned short sw_len;
  $DESCRIPTOR(sw_dsc,sw);

  if (cli$present(c$dsc("BOOL")) & 1) {
    if (cli$get_value(c$dsc("BOOL"),&sw_dsc,&sw_len) & 1) {
      sw[sw_len] = '\0';
      if (!strncmp(sw,"OF",2)) return(set_noline(),0);
      }
    }
  if (line_flag) {
    new_char_buf[2] = edit_on_buf[2];
    if (line_state) {
      edit_m_on();
      line_state = 1;
      }
    line_flag = 0;
    }
  return(0);
}

int set_noline()
{
  if (!line_flag) {
    new_char_buf[2] &= ~(TT2$M_EDITING);
    line_flag = 1;
    if (line_state) edit_m_off();
    }
  return(0);
}

void no_broad_trap()
{
  b_trap_state = 0;
  if (b_trap_flag) {
    smg$disable_broadcast_trapping(&pid);
    sys$qiow(0,chan,IO$_SENSEMODE,iosb,0,0,
	     brd_char_buf,sizeof brd_char_buf,0,0,0,0);
    brd_char_buf[1] &= ~(TT$M_NOBRDCST);
    old_char_buf[1] &= ~(TT$M_NOBRDCST);
    new_char_buf[1] &= ~(TT$M_NOBRDCST);
    edit_on_buf[1] &= ~(TT$M_NOBRDCST);
    if (new_char_buf[0])
      sys$qiow(0,chan,IO$_SETMODE,iosb,0,0,
	       new_char_buf,sizeof new_char_buf,0,0,0,0);
    else
      sys$qiow(0,chan,IO$_SETMODE,iosb,0,0,
	       brd_char_buf,sizeof brd_char_buf,0,0,0,0);
    }
}

void broad_trap()
{
  b_trap_state = 1;
  if (b_trap_flag) {
    smg$set_broadcast_trapping(&pid,b_trap,0);
    sys$qiow(0,chan,IO$_SENSEMODE,iosb,0,0,
	     brd_char_buf,sizeof brd_char_buf,0,0,0,0);
    brd_char_buf[1] |= TT$M_NOBRDCST;
    new_char_buf[1] |= TT$M_NOBRDCST;
    edit_on_buf[1] |= TT$M_NOBRDCST;
    if (new_char_buf[0])
      sys$qiow(0,chan,IO$_SETMODE,iosb,0,0,
	       new_char_buf,sizeof brd_char_buf,0,0,0,0);
    else
      sys$qiow(0,chan,IO$_SETMODE,iosb,0,0,
	       brd_char_buf,sizeof brd_char_buf,0,0,0,0);
    }
}

int set_broad()
{
  char sw[20];
  unsigned short sw_len;
  $DESCRIPTOR(sw_dsc,sw);

  if (cli$present(c$dsc("BOOL")) & 1) {
    if (cli$get_value(c$dsc("BOOL"),&sw_dsc,&sw_len) & 1) {
      sw[sw_len] = '\0';
      if (!strncmp(sw,"OF",2)) return(set_nobroad(),0);
      }
    }
  if (!b_trap_flag) {
    b_trap_flag = 1;
    if (b_trap_state) broad_trap();
    }                                             
  return(0);
}

int set_nobroad()
{
  if (b_trap_flag) {
    if (b_trap_state) {
      no_broad_trap();
      b_trap_state = 1;
      }
    b_trap_flag = 0;
    }
  return(0);
}
