/*
**++
**  FACILITY:
**      NEWSSITE.H
**
**  ABSTRACT:
**      Site tailorable NEWS constant definitions.
**
**  AUTHOR:
**      Geoff Huston
**
**  COPYRIGHT:
**      Copyright  1988,1989,1990,1991,1992
**--
**/

#define NEWS_VERSION    "VMS News - V6.1"
#define NEWS_VDATE      "30-Jan-1993"
#define NEWS_DDATE      "30/1/93"

/*
 *  EXP_TIME
 *  News item lifetime (in days): global default.
 *
 *  GRP_TIME
 *  Newsgroup lifetime (in days): global default.
 */

#define EXP_TIME        14
#define GRP_TIME        60

/*
 *  NEWS_BATCH_SIZE
 *  This is the maximum size of news batch files (in bytes - not blocks)
 */

#define NEWS_BATCH_SIZE 250000

/*
 *  RECHECK_TIMER
 *  The configuration files are periodically re-checked to ensure that any edit
 *  changes made to the files are read into all active NEWS sessions. The timer
 *  determines the frequency of the check of the modification date of these
 *  files. Units are seconds.
 */

#define RECHECK_TIMER   (15 * 60)

/*
 *  DIST_PATH_CHECK
 *  If DIST_PATH_CHECK is defined as 1 then NEWS will check the Path: header of all
 *  incoming news items to see if a neighbouring node has already postmarked
 *  the item, and if so the item will not be forwarded to that node. This
 *  relies on unique node names throughout the network - if this is not the
 *  case then the Path: check can be disabled by setting this value to '0' -
 *  the drawback is a (probably marginal) increase in redundant network traffic.
 */

#ifndef SELF_PATH_CHECK
#define SELF_PATH_CHECK 0
#endif

#ifndef DIST_PATH_CHECK
#define DIST_PATH_CHECK  1
#endif

/*
 *  CANCELMAIL determines whether the USENET account will be mailed in response
 *  to an incoming "cancel" control message. The distribution version sets this
 *  variable to 0 (no auto mail generation for cancel messages).
 */

#ifndef CANCELMAIL
#define CANCELMAIL  0
#endif

/*
 *  UNKCONTROLMAIL determines whether the USENET account will be mailed in
 *  response to an incoming control message with an unrecognised keyword.
 */

#ifndef UNKCONTROLMAIL
#define UNKCONTROLMAIL  1
#endif

/*
 * JUNKNODOT filters incoming news items and removes any newsgroups from the
 * item's Newsgroups: header which do not contain a '.' character. The implicit
 * assumption in USENET NEWS is that only those newsgroups local to an
 * individual node will not have a '.' - all other newsgroups will contain
 * at least one '.' character. However there is a certain amount of abuse of this
 * and it's kind of aysilly distribution limitation mechanism - hence the shipped
 * default is 0. You may wish to set it to 1 (as per the RFC - don't accept
 * incoming newsgroups without a '.') but there's no point in adhering to a written
 * standard if noone else does!
 */

#ifndef JUNKNODOT
#define JUNKNODOT   0
#endif

/*
 * DISTFILE is the flag to determine the way in which the news.distribution and
 * the news.sys file are linked. Incoming news items (not local postings) are
 * searched for the Path: control header, and the leftmost entry in the path
 * string is used as the sending node. This node is looked up in the dist file
 * and the incoming mail is sent only to those nodes listed in the dist file
 * entry which have a matching sys filter. If DISTFILE is 0, then if the node
 * entry is not located in the dist file then the sysfile is used against all
 * nodes. If DISTFILE is 1 then if the node is not in the dist file, then the
 * item is NOT forwarded to any neighbouring nodes
 */

#ifndef DISTFILE
#define DISTFILE    0
#endif

/*
 * UNAPPROVED NEWS is used to define the action when the local site is fed
 * (over the news network) a posting to a moderated newsgroup which is an
 * unapproved item (no Approved: header), and where the local system manager
 * has not specified that approval is optional. The possible actions are to
 * discard the item (default action), accept the item, or mail the item to
 * the moderator. In the case of converted mailing lists into NEWS the last
 * option may lead to mail loops. Note that this does NOT refer to local
 * postings, where the posting is ALWAYS mailed to the first approval
 * moderator address.
 */

#define DISCARD		  0
#define ACCEPT		  1
#define MAIL_TO_MODERATOR 2

#ifndef UNAPPROVED_NEWS
#define UNAPPROVED_NEWS   DISCARD
#endif

/* MIN_DAYS, MAX_DAYS define the acceptable range (in days) of item retention
 * These values are used in modifying the Exipres: information in the newsitem
 * header so that the item retention time lies in the range of these two
 * values.
 * ITMLIFE_DEFAULTS if non-zero indicates that news item life times are further
 * restricted to be no greater than the default item retention period for the
 * newsgroup into which the item is placed.
 */

#ifndef MIN_DAYS
#define MIN_DAYS    4
#endif

#ifndef MAX_DAYS
#define MAX_DAYS    366
#endif

#ifndef ITMLIFE_DEFAULTS
#define ITMLIFE_DEFAULTS   0
#endif

/* STARTLINE

   If STARTLINE is #defined to 1 then you can use the command line of NEWS
   as the first command of an interactive session.. e.g.

     announce :== 'news select local.messages

   which will execute the command line, then continue with an interactive
   session as normal.

   If STARTLINE is set to 0 then if the command line has a command, then
   that single command will be executed and then NEWS will exit
   (This behaviour is similar to the VMS MAIL command)

   The STARTLINE code was supplied by :
		Jonathan Welch
		VAX Systems Programmer   Bitnet: JHWELCH@umaecs
		Univ. of Mass, Amherst   Internet: JHWELCH@ecs.umass.edu

*/

#ifndef STARTLINE
#define STARTLINE	1
#endif

/* NEWS_INIT

   IF NEWS_INIT if #defined to 1, then the logical name "NEWS_INIT" or the
   file "sys$login:news.init" is opened at the start of the news session,
   and the contents of the file are treated as interactive commands.

   NOTE that all interactive prompts relating to subcommands and options
   will stay as interactive prompts - so this is NOT exactly the same
   as full input redirection.

   Any command line options are executed AFTER NEWS_INIT.

*/

#ifndef NEWS_INIT
#define NEWS_INIT	1
#endif

/* DEBUG

   Used to debug ADD FILE - as the output gets verbose, this one is best left
   at 0

*/

#ifndef DEBUG
#define DEBUG	0
#endif

/* MAXUNK
   
   Array size definition used in parsing news item headers. This variable
   refers to the maximum stored number of header lines which are not headers
   recognised by NEWS. 100 is adequate.

*/

#ifndef MAX_UNK
#define MAX_UNK  100
#endif

/* VMSV4, VMSV5 are used in compilation to determine if NEWS is to be linked
 * and executed on a VMS V4 or V5 system. If neither are defined then the
 * compiled code may be linked and executed on either system (but will give
 * link errors on a V4 system because of static links to the V5 callable
 * mail interface
 */

#define VMSV5	1

/* FULL_DIRECTORY_ACCESS_CHECK
   DIRECTORY_ACCESS_CHECK

   If FULL_DIRECTORY_ACCESS_CHECK is defined to 1, then each startup of NEWS
   will check for read access to the directory associated with each newsgroup
   (which can be maintained by SET ACCESS, or direct editing of the ACLS
   attached to a newsgroup).

   If DIRECTORY_ACCESS_CHECK is defined to 1, then the directory check will be
   performed for those newsgroups which have be set RESTRICTed only.

   If both compile variables are defined to 0, then the directory check is
   only performed in the absence of an ACCESS list on only RESTRICTED
   newsgroups.

   NEWS is shipped with both variables set to 0, as this is the fastest
   execution startup option.
*/

#ifndef DIRECTORY_ACCESS_CHECK
#define DIRECTORY_ACCESS_CHECK 0
#endif

#ifndef FULL_DIRECTORY_ACCESS_CHECK
#define FULL_DIRECTORY_ACCESS_CHECK 0
#endif

#if FULL_DIRECTORY_ACCESS_CHECK
#define DIRECTORY_ACCESS_CHECK 0
#endif

/*
 *  NOALLOW_USER_FROM
 *
 * If defined then NEWS will not attempt to substitute a from: mailing address
 * for the default of username@node.domain.
 *
 * If not defined then NEWS will use NEWS_FROM, or if not defined, PMDF_FROM,
 * as the user's mail address. While this would normally only affect other
 * users trying to mail responses to the user, this also has ramifications
 * if you are running a large system where you may have troublesom users..
 * using NEWS_FROM and/or PMDF_FROM users may masquerade as others, and 
 * post items in others' names and cancel others' postings.
 *
 * So think about whether the default setting, implying a trusted user population,
 * is appropriate for your system before accepting this default setting.
 */

#ifndef NOALLOW_USER_FROM
#define NOALLOW_USER_FROM 0
#endif
#if  !defined(ALLOW_USER_FROM) && !NOALLOW_USER_FROM
#define ALLOW_USER_FROM		1
#else
#define ALLOW_USER_FROM		0
#endif

#define NNTP_CACHE  3

/*
 * NEWGROUP_REG is the default value for users being automatically registered
 * into new newsgroups. This is overridden by a SET PROFILE/NEWREGISTER
 * setting. The default distributed value of this is 0 (do not automatically
 * register), but sites MAY prefer to have users know about new newsgroups
 * by registering them automatically, using a value of 1. (Strictly speaking
 * a value between 1 and 255 is possible as this value is the registered
 * priority of the new newsgroup, but it is not recommended.)
 */

#ifndef NEWGROUP_REG
#define NEWGROUP_REG   0
#endif

#ifndef NODE_PATH
#define NODE_PATH	0
#endif

#define USENET  "usenet"

/* */
#define LOG_TABLE   "LNM$SYSTEM"
#define MODE        PSL$C_EXEC
/*  The following are used for testing only .....
#define LOG_TABLE   "LNM$PROCESS"
#define MODE        PSL$C_SUPER
/**/

/*
 * CONFIRM_NEW_GROUPS is defined to 1 to manually ask the user to confirm 
 * registration in new newsgroups which have been created since the
 * previous run of NEWS.
 */

#define CONFIRM_NEW_GROUPS 1

/*
 * NNTP_USE_XHDR determines whether the nntp client will use xhdr commands
 * against the remote server. For servers which do not support this, the use
 * of xhdr will fill the log file with invalid command entries.
 */

#define NNTP_USE_XHDR 1

/*
 * NNTP_USE_XUPDGROUP determines whether the nntp client will use the vms
 * anu-news server-defined xupdgroup command. For non-vms servers set this to
 * 0.
 */

#define NNTP_USE_XUPDGROUP 1

/*
 * NNTP_USE_XGTITLE determines whether the nntp client will use the vms
 * anu-news server-defined xgtitle command. For non-vms servers set this to
 * 0.
 */

#define NNTP_USE_XGTITLE 1

/*
 * NNTP_USE_XGNOTICE determines whether the nntp client will use the vms
 * anu-news server-defined xgnotice command. For non-vms servers set this to
 * 0.
 */

#define NNTP_USE_XGNOTICE 1

/*
 * DISPLAY_BY_NEWSGROUPS determines whether to break and display a newsgroup
 * directory whenever the newsgroup is automatically changed by the item reader.
 */

#define DISPLAY_BY_NEWSGROUPS 1

/*
 * STRIP_ERR_LINES determines whether the CLI% prefixes from the CLI messages are stripped by the
 * code - default is ON
 */

#define STRIP_ERR_LINES 1

/*
 * MENU_SUPPORT determines whether on-screen menu drivers are included into the object
 * default is ON.
 */

#define MENU_SUPPORT	1

/*
 * ARBITRON will cause NEWS to write the date of last item read access to
 * each newsgroup into the user's newsrc file. This will make the arbitron
 * program more accurate in reporting local readership levels.
 * default is ON.
 */

#define ARBITRON	1

/*
 * FAST_LOAD determines whether fast loading drivers are included into the object
 * default is ON. FAST_LOAD requires Arbitron support as well.
 */

#define FAST_LOAD	1
#if FAST_LOAD
#define ARBITRON	1
#endif

/*
 * UWO adds support for UWO/Mail for Vax/VMS 
 * default is OFF
 */
#ifndef UWO
#define UWO		0
#endif

#define USEDOTNEWSRC	0

/*
 * CACHE option for NNTp to use a memory-based cache of message-IDS for faster
 * processing. Turned OFF by default
 *
 * To install cache option:
 * - set the USECACHE flag to 1
 *
 * - rebuild the executables with the USECACHE symbol defined
 *
 * - At startup, define the logical NEWS_CACHE (on one node only!) :
 *   $ DEFINE/SYSTEM/EXEC NEWS_CACHE "Now is the time" ! Must exist for News cache
 *
 * - How to create the cache :
 *   $ mcr sysgen
 *   create news_root:news.cache/size=1400
 *   $ set proc/priv=sysgbl
 *   $ run cachem
 *   Z ! to reset the cache
 */

#ifndef USECACHE
#define USECACHE 1
#endif

#define NOTE_RANGE_MAX	1000	 /* number of slots available for notes ranges */
